<?php
/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

 /**
 * \file AnxNPT.php
 * \author Chris Ford
 * \date January 2006
 */

/**
 * \class AnxNPT
 * A class for encapsulating intervals specified with the npt time scheme.
 */
class AnxNPT extends AnxInterval {

  /** The prefix that distinguishes npt intervals. */
  const PREFIX = "npt:";
  
  /**
   * The constructor.
   *
   * \param $string a string specifying an npt interval.
   *
   * \return TRUE on success, and throws an AnxBadFormatException if $string is
   * not properly formatted.
   */
  public function __construct( $string ) {
    parent::__construct( $string );

    $n = parent::NUMBER;

    /* Strings matching the various kinds of npt strings. */
    $plain_open = self::PREFIX."$n";
    $plain_closed = self::PREFIX."$n-$n";
    $full_open = self::PREFIX."$n:$n:$n";
    $full_closed = self::PREFIX."$n:$n:$n-$n:$n:$n";
  
    /* Match the bits of the time and convert to seconds. */  
    if( $tokens = parent::match( $string, $plain_open ) ) {
      $start = $tokens[0];
      $end = parent::DEFAULT_END;
    } else if( $tokens = parent::match( $string, $plain_closed ) ) {
      $start = $tokens[0];
      $end = $tokens[1];
    } else if( $tokens = parent::match( $string, $full_open ) ) {
      $start = 60*60*$tokens[0] + 60*$tokens[1] + $tokens[2];
      $end = parent::DEFAULT_END;
    } else if( $tokens = parent::match( $string, $full_closed ) ) {
      $start = 60*60*$tokens[0] + 60*$tokens[1] + $tokens[2];
      $end = 60*60*$tokens[3] + 60*$tokens[4] + $tokens[5];
    } else {
      throw new AnxBadFormatException("Bad npt time: "
            ."'$string'");
    }

    $this->set_start($start);
    $this->set_end($end);
    
    return true;
    
  }
}

