/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "phpanx_util.h"
#include "phpanx_types.h"

static zval* add_string_attribute( zval* object, const char* attribute_name, 
		const char* string TSRMLS_DC );
static char* get_assoc_string( zval* array, char* key );
zval* anxlist_to_array( ToObjectConverter make_object, AnxList* list 
		 TSRMLS_DC );
zval* meta_to_object( AnxMetaElement *meta TSRMLS_DC );
zval* link_to_object( AnxLinkElement *link TSRMLS_DC );
		
/* head_to_object() (indirectly) calls add_property_string() from the ZEND API
 * to copy the values of AnxHead fields into a zval* object.
 */	
extern zval* head_to_object( AnxHead *head TSRMLS_DC ) {
	
	zval *object = NULL, *meta = NULL, *link = NULL;
	MAKE_STD_ZVAL( object );
	object_init( object );

	/* If we are given NULL, we return NULL as an error indication */
	if( head == NULL ) {
		return( NULL );
	}

	/* Go through the fields of the AnxHead and add attributes to the 
	 * object. NULL fields will be translated to ""s. 
	 */
	add_string_attribute( object, "head_id", head->head_id TSRMLS_CC );
	add_string_attribute( object, "lang", head->lang TSRMLS_CC );
	add_string_attribute( object, "dir", head->dir TSRMLS_CC );
	add_string_attribute( object, "profile", head->profile TSRMLS_CC );
	add_string_attribute( object, "title", head->title TSRMLS_CC );
	add_string_attribute( object, "title_id", head->title_id TSRMLS_CC );
	add_string_attribute( object, "title_lang", head->title_lang 
			TSRMLS_CC );
	add_string_attribute( object, "title_dir", head->title_dir TSRMLS_CC );
	add_string_attribute( object, "base_id", head->base_id TSRMLS_CC );
	add_string_attribute( object, "base_href", head->base_href TSRMLS_CC );

	meta = anxlist_to_array( (ToObjectConverter) meta_to_object, head->meta
			 TSRMLS_CC );
	add_property_zval( object, "meta", meta);

	link = anxlist_to_array( (ToObjectConverter) link_to_object, head->link
			 TSRMLS_CC );
	add_property_zval( object, "link", link);

	return( object );
}

/* This function sets all of the fields of an AnxHead to NULL, then calls
 * head_to_object().
 */
extern zval* make_head_object( TSRMLS_D ) {

	AnxHead empty_head;
	zval *head_object;

	empty_head.head_id = NULL;
	empty_head.lang = NULL;
	empty_head.dir = NULL;
	empty_head.profile = NULL;
	empty_head.title = NULL;
	empty_head.title_id = NULL;
	empty_head.title_lang = NULL;
	empty_head.title_dir = NULL;
	empty_head.base_id = NULL;
	empty_head.base_href = NULL;
	empty_head.meta = NULL;
	empty_head.link = NULL;

	return( head_to_object( NULL TSRMLS_CC ) );
}

extern AnxHead* object_to_head( zval *object ) {

	AnxHead *head;
	zval **zstring, temp;

//FIXME The memory allocation in this function is nanged
	
	/* If we are given NULL, we return NULL as an error indication */
	if( object == NULL ) {
		return( NULL );
	}

	head = malloc( sizeof(AnxHead) );
	if (head == NULL) {
		return( NULL );
	}

	/* Only the functions for retrieving array elements were documented
	 * at time of coding, so I just convert the object to an array (which
	 * should be a cheap operation).
	 */
	convert_to_array( object );	

	head->head_id = get_assoc_string( object, "head_id" );
	head->lang = get_assoc_string( object, "lang" );
	head->dir = get_assoc_string( object, "dir" );
	head->profile = get_assoc_string( object, "profile" );
	head->title = get_assoc_string( object, "title" );
	head->title_id = get_assoc_string( object, "title_id" );
	head->title_lang = get_assoc_string( object, "title_lang" );
	head->title_dir = get_assoc_string( object, "title_dir" );
	head->base_id = get_assoc_string( object, "base_id" );
	head->base_href = get_assoc_string( object, "base_href" );

	/* Switch the object back */
	convert_to_object( object );

	return( head );
}

PHP_FUNCTION( object_test ) {

	zval *obj, *object, *array, *null, **zvalue = NULL, temp;
	AnxHead head, *second_head;
	char *key, *value, *string;
	int key_length, value_length;
	void** dummy;
	HashTable *obj_hash;
	int obj_count;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zss", 
			&obj, &key, &key_length, &value, &value_length )  
			== FAILURE) {
		RETURN_NULL();
	}

/*
	object_init( return_value );
	//MAKE_STD_ZVAL( return_value );
	//object_init( return_value );
	object_init( obj );
	

//	return_value = obj; 
	RETURN_TRUE;
*/
	object_init(obj);
//	object_init(return_value);
	head.head_id = NULL;
	head.lang = NULL;
	head.dir = "da dum";
	head.profile = "da dum";
	head.title = "da dum";
	head.title_id = "da dum";
	head.title_lang = "da dum";
	head.title_dir = "da dum";
	head.base_id = "base id";
	head.base_href = "da dum";
	head.meta = NULL;
	head.link = NULL;
//	object_init( return_value );
//	add_property_long( obj, "num", 33 );
//	add_property_bool( obj, "big", 33 );
//	head_to_object( return_value, head );
	object = head_to_object( &head TSRMLS_CC );
//	add_property_stringl( obj, "head_id", "fire",5, DUPLICATE );
	//*return_value = *obj;
	//zval_copy_ctor( retun_value );
	null = NULL;

	string = "dir";	
	convert_to_array(object);
	obj_hash = Z_ARRVAL_P(object);
	if( obj_hash == NULL ) {
		php_printf("hashtable is NULL!!!");
	}
	obj_count = zend_hash_num_elements( obj_hash );
//	php_printf("There are %d entries<br>", obj_count);
//	if (zend_hash_find( Z_ARRVAL_P(object), string, strlen(string) + 1, 
//				(void**) &zvalue ) == SUCCESS) {
//		php_printf("success<br>");
//		temp = **zvalue;
//		convert_to_string( &temp );
//		php_printf("<br>The dir value is%s<br><br>", temp);
//	} 	
	convert_to_object( object );
//	array_init(array);
//	switch(Z_TYPE_P( object )) {
//		case IS_NULL:
//			php_printf("It's null");
//			break;
//		case IS_OBJECT:
//			php_printf("It's an object");
//			break;
//		case IS_ARRAY:
//			php_printf("It's an array");
//			break;
//	}

	second_head = object_to_head( object );
	if( second_head == NULL ) {
		php_printf("second head is NULL<br>");
	}
	//RETURN_TRUE;
//	php_printf("<br>base href is %s<br>", second_head->base_href);
//	php_printf("title string is \"%s\"!<br>", second_head->title);
//	php_printf("dir string is \"%s\"!<br>", second_head->dir);
	ANX_RETURN_OBJECT( object );
}

/* Add an attribute to the object, setting it to empty string if the original
 * pointer is NULL. This is essentially a safer wrapper to add_property_string()
 *
 * This function assumes that the names of the object attribute and the struct
 * field will be the same.
 */
static zval* add_string_attribute( zval* object, const char* attribute_name, 
		const char* string TSRMLS_DC ) {

	if( string != NULL ) {
		add_property_string( object, attribute_name, string,
			DUPLICATE );
	} else {
		add_property_string( object, attribute_name, "",
			DUPLICATE );
	}

	return( object );
}

/* Returns the appropriate string attribute from the associative array. 
 *
 * Returns NULL if attribute is not found.
 */  
static char* get_assoc_string( zval* array, char* key ) {

	zval **zstring, temp;
	if (zend_hash_find( Z_ARRVAL_P(array), key, strlen(key) + 1, 
				(void**) &zstring) == FAILURE) {
		return( NULL );
	} else {
		temp = **zstring;
		convert_to_string( &temp );
		return( Z_STRVAL(temp) ); 
	}

}	

int anx_to_resource_id( ANNODEX* anx ) {

	zval* anx_resource;
	int resource_id;

	MAKE_STD_ZVAL( anx_resource );

	resource_id = ZEND_REGISTER_RESOURCE( anx_resource, anx, le_anx_handle);

	return( resource_id );
}

zval* anxlist_to_array( ToObjectConverter make_object, AnxList* list 
		TSRMLS_DC ) {

	zval *array, *object;

	MAKE_STD_ZVAL( array );
	array_init( array );	

	if( list == NULL ) {
		return( array );
	}

	//FIXME This loop may go forever. Perhaps should be a for loop?
	while( !anx_list_is_empty( list ) ) {

		object = make_object( list->data TSRMLS_CC );
		add_next_index_zval( array, object );    
		list = list->next;

	}

	php_printf("list has %d elements<br>", anx_list_length( list ));
	return( array );
}

zval* meta_to_object( AnxMetaElement *meta TSRMLS_DC ) {

	zval *object = NULL;
	MAKE_STD_ZVAL( object );
	object_init( object );

	/* If we are given NULL, we return NULL as an error indication */
	if( meta == NULL ) {
		return( NULL );
	}

	/* Go through the fields of the AnxMetaElement and add attributes to the
	 * object. NULL fields will be translated to ""s. 
	 */
	add_string_attribute( object, "id", meta->id TSRMLS_CC );
	add_string_attribute( object, "lang", meta->lang TSRMLS_CC );
	add_string_attribute( object, "dir", meta->dir TSRMLS_CC );
	add_string_attribute( object, "name", meta->name TSRMLS_CC );
	add_string_attribute( object, "content", meta->content TSRMLS_CC );
	add_string_attribute( object, "scheme", meta->scheme TSRMLS_CC );

	return( object );
	
}

AnxMetaElement* object_to_meta( zval* object ) {

	AnxMetaElement *meta;
	zval **zstring, temp;

	/* If we are given NULL, we return NULL as an error indication */
	if( object == NULL ) {
		return( NULL );
	}

	meta = malloc( sizeof(AnxMetaElement) );
	if (meta == NULL) {
		return( NULL );
	}

	/* Only the functions for retrieving array elements were documented
	 * at time of coding, so I just convert the object to an array (which
	 * should be a cheap operation).
	 */
	convert_to_array( object );	

	meta->id = get_assoc_string( object, "id" );
	meta->lang = get_assoc_string( object, "lang" );
	meta->dir = get_assoc_string( object, "dir" );
	meta->name = get_assoc_string( object, "name" );
	meta->content = get_assoc_string( object, "content" );
	meta->scheme = get_assoc_string( object, "scheme" );

	/* Switch the object back */
	convert_to_object( object );

	return( meta );
}


zval* link_to_object( AnxLinkElement *link TSRMLS_DC ) {

	zval *object = NULL;
	MAKE_STD_ZVAL( object );
	object_init( object );

	/* If we are given NULL, we return NULL as an error indication */
	if( link == NULL ) {
		return( NULL );
	}

	/* Go through the fields of the AnxMetaElement and add attributes to the
	 * object. NULL fields will be translated to ""s. 
	 */
	add_string_attribute( object, "id", link->id TSRMLS_CC );
	add_string_attribute( object, "class", link->class TSRMLS_CC );
	add_string_attribute( object, "title", link->title TSRMLS_CC );
	add_string_attribute( object, "lang", link->lang TSRMLS_CC );
	add_string_attribute( object, "dir", link->dir TSRMLS_CC );
	add_string_attribute( object, "href", link->href TSRMLS_CC );
	add_string_attribute( object, "type", link->type TSRMLS_CC );
	add_string_attribute( object, "rel", link->rel TSRMLS_CC );
	add_string_attribute( object, "rev", link->rev TSRMLS_CC );
	add_string_attribute( object, "media", link->media TSRMLS_CC );

	return( object );
	
}

AnxLinkElement* object_to_link( zval* object ) {

	AnxLinkElement *link;
	zval **zstring, temp;

	/* If we are given NULL, we return NULL as an error indication */
	if( object == NULL ) {
		return( NULL );
	}

	link = malloc( sizeof(AnxLinkElement) );
	if (link == NULL) {
		return( NULL );
	}

	/* Only the functions for retrieving array elements were documented
	 * at time of coding, so I just convert the object to an array (which
	 * should be a cheap operation).
	 */
	convert_to_array( object );	

	link->id = get_assoc_string( object, "id" );
	link->class = get_assoc_string( object, "class" );
	link->title = get_assoc_string( object, "title" );
	link->lang = get_assoc_string( object, "lang" );
	link->dir = get_assoc_string( object, "dir" );
	link->href = get_assoc_string( object, "href" );
	link->type = get_assoc_string( object, "type" );
	link->rel = get_assoc_string( object, "rel" );
	link->rev = get_assoc_string( object, "rev" );
	link->media = get_assoc_string( object, "media" );

	/* Switch the object back */
	convert_to_object( object );

	return( link );
}

zval* clip_to_object( AnxClip *clip TSRMLS_DC ) {

	zval *meta;
	zval *object = NULL;
	MAKE_STD_ZVAL( object );
	object_init( object );

	/* If we are given NULL, we return NULL as an error indication */
	if( clip == NULL ) {
		return( NULL );
	}

	/* Go through the fields of the AnxMetaElement and add attributes to the
	 * object. NULL fields will be translated to ""s. 
	 */
	add_string_attribute( object, "clip_id", clip->clip_id TSRMLS_CC );
	add_string_attribute( object, "class", clip->class TSRMLS_CC );
	add_string_attribute( object, "title", clip->title TSRMLS_CC );
	add_string_attribute( object, "lang", clip->lang TSRMLS_CC );
	add_string_attribute( object, "dir", clip->dir TSRMLS_CC );
	add_string_attribute( object, "track", clip->track TSRMLS_CC );
	add_string_attribute( object, "anchor_id", clip->anchor_id TSRMLS_CC );
	add_string_attribute( object, "anchor_class", clip->anchor_class 
			 TSRMLS_CC );
	add_string_attribute( object, "anchor_title", clip->anchor_title 
			 TSRMLS_CC );
	add_string_attribute( object, "anchor_lang", clip->anchor_lang 
			 TSRMLS_CC );
	add_string_attribute( object, "anchor_dir", clip->anchor_dir 
			 TSRMLS_CC );
	add_string_attribute( object, "anchor_href", clip->anchor_href 
			 TSRMLS_CC );
	add_string_attribute( object, "anchor_text", clip->anchor_text 
			 TSRMLS_CC );
	add_string_attribute( object, "img_id", clip->img_id TSRMLS_CC );
	add_string_attribute( object, "img_class", clip->img_class TSRMLS_CC );
	add_string_attribute( object, "img_title", clip->title TSRMLS_CC );
	add_string_attribute( object, "img_lang", clip->img_lang TSRMLS_CC );
	add_string_attribute( object, "img_dir", clip->dir TSRMLS_CC );
	add_string_attribute( object, "img_src", clip->img_src TSRMLS_CC );
	add_string_attribute( object, "img_alt", clip->img_alt TSRMLS_CC );
	add_string_attribute( object, "desc_id", clip->desc_id TSRMLS_CC );
	add_string_attribute( object, "desc_class", clip->desc_class 
			 TSRMLS_CC );
	add_string_attribute( object, "desc_title", clip->desc_title 
			 TSRMLS_CC );
	add_string_attribute( object, "desc_lang", clip->desc_lang TSRMLS_CC );
	add_string_attribute( object, "desc_dir", clip->desc_dir TSRMLS_CC );
	add_string_attribute( object, "desc_text", clip->desc_text TSRMLS_CC );

	meta = anxlist_to_array( (ToObjectConverter) meta_to_object, clip->meta
		        TSRMLS_CC );
	add_property_zval( object, "meta", meta);

	return( object );
	
}

AnxClip* object_to_clip( zval* object ) {

	AnxClip *clip;
	zval **zstring, temp;

	/* If we are given NULL, we return NULL as an error indication */
	if( object == NULL ) {
		return( NULL );
	}

	clip = malloc( sizeof(AnxClip) );
	if (clip == NULL) {
		return( NULL );
	}

	/* Only the functions for retrieving array elements were documented
	 * at time of coding, so I just convert the object to an array (which
	 * should be a cheap operation).
	 */
	convert_to_array( object );	

	clip->clip_id = get_assoc_string( object, "clip_id" );
	clip->class = get_assoc_string( object, "class" );
	clip->title = get_assoc_string( object, "title" );
	clip->lang = get_assoc_string( object, "lang" );
	clip->dir = get_assoc_string( object, "dir" );
	clip->track = get_assoc_string( object, "track" );
	clip->anchor_id = get_assoc_string( object, "anchor_id" );
	clip->anchor_class = get_assoc_string( object, "anchor_class" );
	clip->anchor_title = get_assoc_string( object, "anchor_title" );
	clip->anchor_lang = get_assoc_string( object, "anchor_lang" );
	clip->anchor_dir = get_assoc_string( object, "anchor_dir" );
	clip->anchor_href = get_assoc_string( object, "anchor_href" );
	clip->anchor_text = get_assoc_string( object, "anchor_text" );
	clip->img_id = get_assoc_string( object, "img_id" );
	clip->img_class = get_assoc_string( object, "img_class" );
	clip->img_title = get_assoc_string( object, "img_title" );
	clip->img_lang = get_assoc_string( object, "img_lang" );
	clip->img_dir = get_assoc_string( object, "img_dir" );
	clip->img_src = get_assoc_string( object, "img_src" );
	clip->img_alt = get_assoc_string( object, "img_alt" );
	clip->desc_id = get_assoc_string( object, "desc_id" );
	clip->desc_class = get_assoc_string( object, "desc_class" );
	clip->desc_title = get_assoc_string( object, "desc_title" );
	clip->desc_lang = get_assoc_string( object, "desc_lang" );
	clip->desc_dir = get_assoc_string( object, "desc_dir" );
	clip->desc_text = get_assoc_string( object, "desc_text" );

	/* Switch the object back */
	convert_to_object( object );

	return( clip );
}

